import React, { useEffect, useState } from "react";
import {
  getAllStageTypesFromAPI,
  getAllTeamsFromAPI,
  createNewRelayToAPI,
  editRelayToAPI,
} from "../../utils/apiRequests";
import validateRelay from "../../utils/validateRelay";
import { useNavigate } from "react-router-dom";

// components
import RelayInfo from "./components/RelayInfo";
import RelayStages from "./components/RelayStages";
import AddTeams from "./components/AddTeams";
import CardBox from "../../components/CardBox";
import BackNextButton from "./components/BackNextButton";
import ConfirmModal from "../../components/ConfirmModal";
import Loader from "../../components/Loader";
import MessageBox from "../../components/MessageBox";

// mui
import { makeStyles } from "@mui/styles";
import Stepper from "@mui/material/Stepper";
import Step from "@mui/material/Step";
import StepLabel from "@mui/material/StepLabel";
import { DirectionsRun, DirectionsBike } from "@mui/icons-material";

const steps = ["Løpsinformasjon", "Antall etapper", "Legg til lag"];

const useStyles = makeStyles((theme) => ({
  stepper: {
    width: "70%",
    maxWidth: "1000px",
    margin: "0 auto",
    marginTop: "20px",
    "@media (max-width: 780px)": {
      display: "none",
    },
  },
  centerItemsContainer: {
    display: "flex",
    alignItems: "center",
    flexDirection: "column",
    marginTop: "20px",
  },
  addRelayStageButtonContainer: {
    display: "flex",
    flexDirection: "column",
    alignItems: "center",
    marginBottom: "30px",
  },
}));

const NewRelay = ({ editMode, relayData, relayId }) => {
  const classes = useStyles();
  const navigate = useNavigate();
  const [activeStep, setActiveStep] = useState(0);
  const [locationStartName, setLocationStartName] = useState("");
  const [relayInformationText, setRelayInformationText] = useState("");
  const [relayStartDate, setRelayStartDate] = useState(null);
  const [relayStartTime, setRelayStartTime] = useState(null);
  const [radius, setRadius] = useState(190);
  const [pushInterval, setPushInterval] = useState(10);
  const [durationInHours, setDurationInHours] = useState(0);
  const [startLatitude, setStartLatitude] = useState(0);
  const [startLongitude, setStartLongitude] = useState(0);
  const [openConfirmDialog, setOpenConfirmDialog] = useState(false);
  const [relayStages, setRelayStages] = useState([]);
  const [circularRelay, setCircularRelay] = useState(true);
  const [stageTypes, setStageTypes] = useState([]);
  const [teams, setTeams] = useState([]);
  const [isLoading, setIsLoading] = useState(true);
  const [messageBox, setMessageBox] = useState({
    message: "",
    severity: "error",
  });

  useEffect(() => {
    getStageTypes()
      .then(() => getAllTeams().then(() => setIsLoading(false)))
      .catch((error) =>
        setMessageBox({
          message: error.message,
          severity: "error",
        })
      );
  }, []);

  useEffect(() => {
    // if in edit mode and teams array does not have checked (teams has then not been initialize with teams data from api)
    if (
      editMode &&
      teams.length > 0 &&
      !teams.some((obj) => Object.keys(obj).includes("checked"))
    ) {
      setLocationStartName(relayData.location);
      setRelayInformationText(relayData.information);
      let startDatePlaceholder = new Date(relayData.startTime);
      let endDatePlaceholder = new Date(relayData.endTime);
      setRelayStartDate(startDatePlaceholder);
      setRelayStartTime(startDatePlaceholder);
      setDurationInHours(
        Math.abs(endDatePlaceholder - startDatePlaceholder) / 36e5
      );
      setRadius(relayData.exchangeRadiusMeters);
      setPushInterval(relayData.frontendUpdateSeconds);
      setStartLatitude(relayData.latitude);
      setStartLongitude(relayData.longitude);
      setCircularRelay(true); // none circular relay is not supported yet
      // relay stages
      const relayStagesPlaceholder = relayData.relayStages.map((rs) => {
        return {
          stageTypeIndex: stageTypes.map((e) => e.id).indexOf(rs.stageTypeId),
          lat: rs.endLatitude,
          lng: rs.endLongitude,
        };
      });
      setRelayStages(relayStagesPlaceholder);
      // teams selected
      const newTeamsPlaceholder = teams.map((team) => {
        let teamPlaceholder = { ...team };
        if (relayData.teams.some((e) => e.id === team.id)) {
          teamPlaceholder.checked = true;
        }
        return teamPlaceholder;
      });
      setTeams(newTeamsPlaceholder);
    }
  }, [stageTypes, relayData, editMode, teams]);

  const getStageTypes = async () => {
    try {
      const stageTypesFromAPI = await getAllStageTypesFromAPI();
      const stageTypesPlaceholder = stageTypesFromAPI.map(
        (stageType, index) => {
          return {
            id: stageType.id,
            name: stageType.category,
            icon:
              index === 0 ? (
                <DirectionsRun fontSize="small" />
              ) : (
                <DirectionsBike fontSize="small" />
              ),
          };
        }
      );
      setStageTypes(stageTypesPlaceholder);
    } catch (error) {
      setMessageBox({
        message: error.message,
        severity: "error",
      });
    }
  };

  const getAllTeams = async () => {
    try {
      const teamsPlaceholder = await getAllTeamsFromAPI();
      setTeams(teamsPlaceholder);
    } catch (error) {
      throw error;
    }
  };

  // removes a stage type from list.
  const removeStageClick = (idx) => {
    const placeholder = [...relayStages];
    const index = placeholder.indexOf(idx);

    placeholder.splice(index, 1);

    setRelayStages(placeholder);
  };

  const addRelayStage = () => {
    const placeholder = [...relayStages];
    placeholder.push({
      lat: startLatitude,
      lng: startLongitude,
      stageTypeIndex: 1,
    });
    setRelayStages(placeholder);
  };

  const handleNext = () => {
    if (activeStep === steps.length - 1) {
      setOpenConfirmDialog(true);
      //submitRelayToAPI();
    } else {
      setActiveStep((prevActiveStep) => prevActiveStep + 1);
    }
  };

  const handleBack = () => {
    setActiveStep((prevActiveStep) => prevActiveStep - 1);
  };

  const submitRelayToAPI = async () => {
    const startDate = new Date(relayStartDate);
    const startTime = new Date(relayStartTime);
    startDate.setHours(startTime.getHours(), startTime.getMinutes());
    let endDate = new Date(
      startDate.getTime() + durationInHours * 60 * 60 * 1000
    );
    const relayData = {
      latitude: startLatitude,
      longitude: startLongitude,
      location: locationStartName,
      startTime: relayStartTime.toJSON(),
      endTime: endDate.toJSON(),
      information: relayInformationText,
      circularRelay: circularRelay,
      teamIds: teams.filter((team) => team.checked).map((team) => team.id),
      exchangeRadiusMeter: radius,
      frontendUpdateSeconds: pushInterval,
      relayStages: relayStages.map((relayStage, index) => {
        return {
          stageTypeId: stageTypes[relayStage.stageTypeIndex].id,
          stageNumber: index + 1,
          lat: relayStage.lat,
          lng: relayStage.lng,
        };
      }),
    };

    const validRelay = validateRelay(relayData);
    if (!validRelay.isValid) {
      setMessageBox({
        message: validRelay.error,
        severity: "error",
      });
      return;
    }

    if (editMode) {
      await editRelayToAPI(relayData, relayId).then(() => {
        navigate("/relay/" + relayId);
      });
    } else {
      await createNewRelayToAPI(relayData).then((id) => {
        navigate("/relay/" + id);
      });
    }
  };

  return (
    <div>
      <Loader isLoading={isLoading}>
        <CardBox>
          <div style={{ paddingBottom: "40px" }}>
            <Stepper activeStep={activeStep} className={classes.stepper}>
              {steps.map((label) => {
                return (
                  <Step key={label}>
                    <StepLabel>{label}</StepLabel>
                  </Step>
                );
              })}
            </Stepper>
          </div>
          {activeStep === 0 && (
            <RelayInfo
              startLatitude={startLatitude}
              setStartLatitude={setStartLatitude}
              startLongitude={startLongitude}
              setStartLongitude={setStartLongitude}
              locationStartName={locationStartName}
              setLocationStartName={setLocationStartName}
              relayStartDate={relayStartDate}
              setRelayStartDate={setRelayStartDate}
              relayStartTime={relayStartTime}
              setRelayStartTime={setRelayStartTime}
              relayInformationText={relayInformationText}
              setRelayInformationText={setRelayInformationText}
              durationInHours={durationInHours}
              setDurationInHours={setDurationInHours}
              radius={radius}
              setRadius={setRadius}
              pushInterval={pushInterval}
              setPushInterval={setPushInterval}
              editMode={editMode}
            />
          )}
          {activeStep === 1 && (
            <RelayStages
              startLatitude={startLatitude}
              startLongitude={startLongitude}
              addRelayStage={addRelayStage}
              circularRelay={circularRelay}
              setCircularRelay={setCircularRelay}
              relayStages={relayStages}
              setRelayStages={setRelayStages}
              removeStageClick={removeStageClick}
              stageTypes={stageTypes}
            />
          )}
          {activeStep === 2 && <AddTeams teams={teams} setTeams={setTeams} />}
          <BackNextButton
            id="back-next-button"
            disabled={startLatitude === 0 || startLongitude === 0}
            steps={steps}
            activeStep={activeStep}
            handleBack={handleBack}
            handleNext={handleNext}
          />
        </CardBox>
        <ConfirmModal
          openConfirmDialog={openConfirmDialog}
          setOpenConfirmDialog={setOpenConfirmDialog}
          dialogTitle={editMode ? "Oppdater løp" : "Lag nytt løp"}
          dialogText={
            editMode
              ? "Er du sikker på at du vil oppdatere APIet med ny løpsdata?"
              : "Er du sikker på at du vil lage et nytt løp til APIet?"
          }
          confirmFunction={submitRelayToAPI}
        />
      </Loader>
      <MessageBox messageBox={messageBox} setMessageBox={setMessageBox} />
    </div>
  );
};

export default NewRelay;
