import React, { useState, useEffect } from "react";
import AddTeamDialog from "../../components/AddTeamDialog";
import EditOrDeleteTeamDialog from "../../components/EditOrDeleteTeamDialog";
import NestedDeleteTeamDialog from "../../components/NestedDeleteTeamDialog";
import MessageBox from "../../components/MessageBox";
import {
  editTeamAPI,
  deleteTeamApi,
  getAllTeamsFromAPI,
  createNewTeamToAPI,
} from "../../utils/apiRequests";

// mui
import Divider from "@mui/material/Divider";
import CardBox from "../../components/CardBox";
import EditIcon from "@mui/icons-material/Edit";
import Loader from "../../components/Loader";
import { makeStyles } from "@mui/styles";
import { Button } from "@mui/material";
import { List, ListItem, ListItemText, ListItemButton } from "@mui/material";
import AddIcon from "@mui/icons-material/Add";

const useStyles = makeStyles((theme) => ({
  addTeamContainer: {
    display: "flex",
    alignItems: "center",
    flexDirection: "column",
    padding: "50px",
  },
  spaceContainer: {
    display: "flex",
    justifyContent: "space-between",
    marginBottom: "20px",
  },
  listContainer: {
    display: "flex",
    flexDirection: "column",
    margin: "0 auto",
    width: "100%",
    maxWidth: "500px",
    textAlign: "center",
    alignItems: "center",
    height: "450px",
  },
  addIconContainer: {
    display: "flex",
    flexDirection: "column",
    alignItems: "center",
  },
}));

const teamCache = [{ name: "", email: "" }];

const TeamsOverview = () => {
  const classes = useStyles();
  const [name, setNameInput] = useState("");
  const [email, setEmail] = useState("");
  const [listName, setListName] = useState("");
  const [listEmail, setListEmail] = useState("");
  const [teamData, setTeamData] = useState([]);
  const [open, setOpen] = useState(false);
  const [listTeamOpen, setListTeamOpen] = useState(false);
  const [nestedDeleteList, setNestedDeleteList] = useState(false);
  const [selectedIndex, setSelectedIndex] = useState(1);
  const [messageBox, setMessageBox] = useState({
    message: "",
    severity: "error",
  });

  const handleListItemClick = (event, index) => {
    setSelectedIndex(index);
    setListName(teamData[index].teamName);
    setListEmail(teamData[index].email);

    teamCache[0].name = teamData[index].teamName;
    teamCache[0].email = teamData[index].email;
    setListTeamOpen(true);
  };

  // This opens the delete nested dialog.
  const deleteTeamClick = () => {
    setNestedDeleteList(true);
  };

  const nestedDeleteClick = async () => {
    try {
      await deleteTeamApi(teamData[selectedIndex].id);
    } catch (error) {
      setMessageBox({ message: error.message, severity: "error" });
    }
    setNestedDeleteList(false);
  };

  const handleTeamButtonClick = () => {
    setOpen(true);
  };

  const addTeamButtonClick = async () => {
    if (name.length < 1) {
      setMessageBox({
        message: "Kan ikke legge til et lag uten navn",
        severity: "error",
      });
    } else if (!email.includes("@")) {
      setMessageBox({
        message: "Email adressen følger ikke riktig format.",
        severity: "error",
      });
    } else {
      const teamObj = {
        TeamName: name,
        Email: email,
      };
      createNewTeamToAPI(teamObj);
    }

    setNameInput("");
    setEmail("");
    setOpen(false);
  };

  const editDataListClick = async () => {
    if (!listEmail.includes("@")) {
      setMessageBox({
        message: "Email adressen følger ikke riktig format.",
        severity: "error",
      });
      setListTeamOpen(false);
      return;
    }

    if (listName === teamCache[0].name && listEmail === teamCache[0].email) {
      teamCache[0].name = "";
      teamCache[0].email = "";
    } else if (
      listName === teamCache[0].name &&
      listEmail !== teamCache[0].email
    ) {
      const newTeam = {
        Email: listEmail,
      };
      editTeamAPI(teamData[selectedIndex].id, newTeam);
    } else if (
      listName !== teamCache[0].name &&
      listEmail === teamCache[0].email
    ) {
      const newTeam = {
        TeamName: listName,
      };
      editTeamAPI(teamData[selectedIndex].id, newTeam);
    } else {
      const newTeam = {
        TeamName: listName,
        Email: listEmail,
      };
      editTeamAPI(teamData[selectedIndex].id, newTeam);
    }
    setListTeamOpen(false);
  };

  const getAllTeams = async () => {
    try {
      const teamsPlaceholder = await getAllTeamsFromAPI();
      setTeamData(teamsPlaceholder);
    } catch (error) {
      setMessageBox({
        message: error.message,
        severity: "error",
      });
    }
  };

  useEffect(() => {
    getAllTeams();
  }, []);

  return (
    <CardBox>
      <Loader isLoading={teamData.length < 1}>
        <div className={classes.addIconContainer} style={{ padding: "20px" }}>
          <Button
            variant="contained"
            size="medium"
            onClick={handleTeamButtonClick}
            startIcon={<AddIcon fontSize="small" />}
          >
            Legg til lag
          </Button>
        </div>

        <div className={classes.listContainer}>
          <List style={{ overflow: "auto", width: "100%" }}>
            {teamData.map((data, idx) => (
              <>
                <ListItem key={data.teamName}>
                  <ListItemButton
                    onClick={(event) => handleListItemClick(event, idx)}
                  >
                    <ListItemText
                      primary={data.teamName}
                      secondary={data.email}
                    />
                    <EditIcon />
                  </ListItemButton>
                </ListItem>
                <Divider />
              </>
            ))}
          </List>
        </div>
      </Loader>

      <MessageBox messageBox={messageBox} setMessageBox={setMessageBox} />

      <AddTeamDialog
        openDialog={open}
        setOpenDialog={setOpen}
        dialogTitle="Legg til lag"
        sendTeamFunc={addTeamButtonClick}
        valueOne={name}
        valueTwo={email}
        setValueOne={setNameInput}
        setValueTwo={setEmail}
      />

      <EditOrDeleteTeamDialog
        openDialog={listTeamOpen}
        setOpenDialog={setListTeamOpen}
        dialogTitle={"Rediger / Slett"}
        editDataFunc={editDataListClick}
        deleteTeamFunc={deleteTeamClick}
        valueOne={listName}
        valueTwo={listEmail}
        setValueOne={setListName}
        setValueTwo={setListEmail}
      />

      <NestedDeleteTeamDialog
        openDialog={nestedDeleteList}
        setOpenDialog={setNestedDeleteList}
        dialogTitle={"Slett"}
        deleteTeamFunc={nestedDeleteClick}
        listName={listName}
      />
    </CardBox>
  );
};

export default TeamsOverview;
