import "react-native-gesture-handler";
import React, { useEffect, useState } from "react";
import { StyleSheet } from "react-native";
import { StatusBar } from "expo-status-bar";
import { NavigationContainer } from "@react-navigation/native";
import { NativeBaseProvider, Button } from "native-base";
import { createNativeStackNavigator } from "@react-navigation/native-stack";
import { LinearGradient } from "expo-linear-gradient";
import * as TaskManager from "expo-task-manager";
import * as Location from "expo-location";

import Background from "./components/Background";
import Landing from "./pages/Landing";
import Tracking from "./pages/Tracking";
import ToastBox from "./components/ToastBox/index";
import { getData, storeData } from "./utils/storage";
import { authorizeInviteKeyToAPI } from "./utils/apiRequests";
import { postLocationToAPI } from "./utils/apiRequests";

const LOCATION_TASK_NAME = "LOCATION_TASK_NAME";

// Define the background task for location tracking
TaskManager.defineTask(LOCATION_TASK_NAME, async ({ data, error }) => {
  if (error) {
    console.error(error);
    return;
  }
  if (data) {
    // Extract location coordinates from data
    const { locations } = data;
    const location = locations[0];
    if (location) {
      const coords = {
        latitude: location.coords.latitude,
        longitude: location.coords.longitude,
      };
      postLocationToAPI(coords);
    }
  }
});

export default function App() {
  // this is a array and not an object because of the famous state/iteration/rerender situation
  const [toastMessage, setToastMessage] = useState([]);
  const [validToken, setValidToken] = useState("");
  const [displayData, setDisplayData] = useState({
    teamName: "",
    stageNumber: 0,
    stageType: "",
  });

  useEffect(() => {
    (async () => {
      let token = await getData("token");
      if (token !== undefined && token !== "") {
        await startTrackingWithInviteKey(token);
      } else {
        console.log("Could not find token");
      }
    })();
  }, []);

  const startTrackingWithInviteKey = async (token) => {
    try {
      let response = await authorizeInviteKeyToAPI(token);
      setValidToken(token);
      setDisplayData({
        teamName: response.teamName,
        stageNumber: response.stageNumber,
        stageType: response.stageType,
      });
      await storeData(token);
      await startBackgroundUpdate();
    } catch (error) {
      setValidToken("");
      const message = [error.message, "error"];
      setToastMessage([...message]);
    }
  };

  const startBackgroundUpdate = async () => {
    // Don't track position if permission is not granted
    const test = await Location.enableNetworkProviderAsync();
    console.log(test);
    const foreground = await Location.requestForegroundPermissionsAsync();
    if (!foreground.granted) {
      throw new Error("Foreground location permission not granted");
    }
    const background = await Location.requestBackgroundPermissionsAsync();
    if (!background.granted) {
      throw new Error("Background location permission not granted");
    }

    // Make sure the task is defined otherwise do not start tracking
    const isTaskDefined = await TaskManager.isTaskDefined(LOCATION_TASK_NAME);
    if (!isTaskDefined) {
      throw new Error("Admin Error: Task is not defined");
    }

    // Don't track if it is already running in background
    const hasStarted = await Location.hasStartedLocationUpdatesAsync(
      LOCATION_TASK_NAME
    );
    if (hasStarted) {
      await stopBackgroundUpdate();
      throw new Error("Location tracking has already started");
    }

    await Location.startLocationUpdatesAsync(LOCATION_TASK_NAME, {
      // For better logs, we set the accuracy to the most sensitive option
      accuracy: Location.Accuracy.BestForNavigation,
      timeInterval: 5000,
      // Make sure to enable this notification if you want to consistently track in the background
      showsBackgroundLocationIndicator: true,
      foregroundService: {
        notificationTitle: "Location",
        notificationBody: "Location tracking in background",
        notificationColor: "#fff",
      },
    });
  };

  const logout = async () => {
    try {
      setValidToken("");
      await stopBackgroundUpdate();
      await storeData("");
    } catch (error) {
      const message = [error.message, "error"];
      setToastMessage([...message]);
    }
  };

  // Stop location tracking in background
  const stopBackgroundUpdate = async () => {
    const hasStarted = await Location.hasStartedLocationUpdatesAsync(
      LOCATION_TASK_NAME
    );
    if (hasStarted) {
      await Location.stopLocationUpdatesAsync(LOCATION_TASK_NAME);
      setToastMessage([...message]);
    }
  };

  function Navigator() {
    const Stack = createNativeStackNavigator();
    return (
      <NavigationContainer>
        <Stack.Navigator screenOptions={{ headerShown: false }}>
          {Boolean(validToken) ? (
            <Stack.Screen name="Tracking">
              {(props) => (
                <Background>
                  <Tracking
                    {...props}
                    displayData={displayData}
                    logout={logout}
                  />
                </Background>
              )}
            </Stack.Screen>
          ) : (
            <Stack.Screen name="Landing">
              {(props) => (
                <Background>
                  <Landing
                    {...props}
                    setToastMessage={setToastMessage}
                    startTrackingWithInviteKey={startTrackingWithInviteKey}
                  />
                  <Button
                    onPress={() => stopBackgroundUpdate()}
                    style={{
                      marginBottom: 20,
                      width: 300,
                      marginLeft: "auto",
                      marginRight: "auto",
                    }}
                  >
                    Reset data (Trykk her om noe ikke fungerer)
                  </Button>
                </Background>
              )}
            </Stack.Screen>
          )}
        </Stack.Navigator>
      </NavigationContainer>
    );
  }
  return (
    <NativeBaseProvider config={config}>
      <StatusBar style="auto" />
      <ToastBox toastMessage={toastMessage} />
      <Navigator />
    </NativeBaseProvider>
  );
}

const config = {
  dependencies: {
    "linear-gradient": LinearGradient,
  },
};

const styles = StyleSheet.create({
  container: {
    flex: 1,
    width: 200,
    height: 200,
  },
});
