﻿using BikeAndRunAPI.Data;
using BikeAndRunAPI.Hubs;
using BikeAndRunAPI.ViewModels;
using Microsoft.AspNetCore.SignalR;
using Microsoft.Extensions.DependencyInjection;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace BikeAndRunAPI.Jobs
{
    /// <summary>
    /// Contains the driving engine of the SignalR framework we use to send data with Websockets.
    /// Clients (front end) are connected to this websocket via the IHubContext.
    /// Allows us to push data as it arrives to the api downstream
    /// to the frontend which displays it on a google maps view as markers.
    /// </summary>
    public class PushRoutine
    {

        private readonly IHubContext<CoordinatesHub> _hubContext;
        private readonly IServiceScopeFactory _scopeFactory;

        public PushRoutine(IServiceScopeFactory scopeFactory, IHubContext<CoordinatesHub> hubContext)
        {
            _scopeFactory = scopeFactory;
            _hubContext = hubContext;
        }

        /// <summary>
        /// routine for sending coordinates for relay to frontend when a relay has started
        /// </summary>
        /// <param name="seconds">seconds between each coordinate push to frontend</param>
        /// <param name="relayId">id of relay</param>
        public async Task StartPushRoutine(int seconds, long relayId)
        {
            var runLoop = true;
            // loops every n seconds
            while (runLoop)
            {
                System.Diagnostics.Debug.WriteLine(seconds);
                System.Diagnostics.Debug.WriteLine(relayId);
                using (var scope = _scopeFactory.CreateScope())
                {
                    var _context = scope.ServiceProvider.GetRequiredService<BikeAndRunContext>();
                    // gets all participants ids in relay in racing stage
                    var participantsInRaceId = (from participant in _context.Participants
                                                join relayStage in _context.RelayStages on participant.RelayStageId equals relayStage.Id
                                                where relayStage.RelayId.Equals(relayId)
                                                where participant.IsInRacingStage.Equals(true)
                                                select participant.Id
                                        ).ToList();

                    // if no result, then relay has ended. Stop push routine.
                    if (participantsInRaceId.Count < 1)
                    {
                        return;
                    }

                    var coordinates = new List<CoordinatesToClient>();
                    // get the last 4 coordinates for each participant in racing stage
                    foreach (var participantId in participantsInRaceId)
                    {
                        var participantCoordinates = (from coordinate in _context.Coordinates
                                                      join participant in _context.Participants on coordinate.ParticipantId equals participant.Id
                                                      join team in _context.Teams on participant.TeamId equals team.Id
                                                      join relaystage in _context.RelayStages on participant.RelayStageId equals relaystage.Id
                                                      where coordinate.ParticipantId.Equals(participantId)
                                                      orderby coordinate.Timestamp descending
                                                      select new CoordinatesToClient()
                                                      {
                                                          Id = coordinate.Id,
                                                          Latitude = coordinate.Latitude,
                                                          Longitude = coordinate.Longitude,
                                                          Timestamp = coordinate.Timestamp,
                                                          StageNumber = relaystage.StageNumber,
                                                          ParticipantId = participant.Id,
                                                          TeamName = team.TeamName,
                                                          TeamId = team.Id,
                                                      }).ToList().Take(4);

                        coordinates = coordinates.Concat(participantCoordinates).ToList();
                    }
                    // sends coordinates through websocket
                    var connectionString = "Coordinates-RelayId" + relayId;
                    await _hubContext.Clients.All.SendAsync(connectionString, coordinates);
                }
                var delayTask = Task.Delay(1000 * seconds);
                await delayTask;
            }
        }

    }
}
