﻿using BikeAndRunAPI.Data;
using BikeAndRunAPI.Data.Models;
using BikeAndRunAPI.Services;
using Microsoft.Extensions.DependencyInjection;
using System;
using System.Linq;
using System.Threading.Tasks;

namespace BikeAndRunAPI.Jobs
{
    /// <summary>
    /// Sends the signal to the database that is should mark the race as started.
    /// The class also handles the deletion of a relay should it check and it has ended at the time of the check.
    /// </summary>
    public class StartAndEndRaceRoutine
    {
        private readonly IServiceScopeFactory _scopeFactory;
        public RelayService _relayService;
        private readonly int checkIntervalMinutes = 10;

        public StartAndEndRaceRoutine(IServiceScopeFactory scopeFactory, RelayService relayService)
        {
            _scopeFactory = scopeFactory;
            _relayService = relayService;
        }

        public async Task StartRoutineCheck()
        {
            var runLoop = true;
            while (runLoop)
            {
                using (var scope = _scopeFactory.CreateScope())
                {
                    var _context = scope.ServiceProvider.GetRequiredService<BikeAndRunContext>();
                    var today = DateTime.Today;
                    foreach (var relay in _context.Relays.ToList())
                    {
                        _ = EndRelayCheck(today, relay);

                        // if relay did not end. 
                        if (_context.Relays.FirstOrDefault(r => r.Id == relay.Id) != null)
                        {
                            _ = StartRelayCheck(today, relay);
                        }
                    }
                    var delayTask = Task.Delay(1000 * (60 * checkIntervalMinutes));
                    await delayTask;
                };
            }
        }

        /// <summary>
        /// deletes relay if relay end date time is past current local time
        /// </summary>
        /// <param name="today">date today</param>
        /// <param name="relay">relay data</param>
        /// <param name="_context">database</param>
        /// <returns>Threading task</returns>
        private Task EndRelayCheck(DateTime today, Relay relay)
        {
            // if past endtime, delete relay
            if (relay.EndTime < today)
            {
                _relayService.DeleteRelayByRelayId(relay.Id);
            }

            return Task.CompletedTask;
        }

        /// <summary>
        /// starts relay if relay start date time is past current local time
        /// </summary>
        /// <param name="today">date today</param>
        /// <param name="relay">relay data</param>
        /// <param name="_context">database</param>
        /// <returns>Threading task</returns>
        private Task StartRelayCheck(DateTime today, Relay relay)
        {
            TimeSpan timeBetween = relay.StartTime - today;
            var timeBetweenInMinutes = timeBetween.TotalMinutes;
            if (!relay.HasStarted && timeBetweenInMinutes < (checkIntervalMinutes + 1))
            {
                _relayService.StartRelay(relay.Id);
            }
            return Task.CompletedTask;
        }
    }
}
