﻿using BikeAndRunAPI.Data;
using BikeAndRunAPI.Data.DTO;
using BikeAndRunAPI.Data.Models;
using BikeAndRunAPI.Hubs;
using BikeAndRunAPI.Scripts;
using BikeAndRunAPI.ViewModels;
using Microsoft.AspNetCore.SignalR;
using System;
using System.Collections.Generic;
using System.Linq;

namespace BikeAndRunAPI.Services
{
    public class CoordinateService
    {
        private readonly BikeAndRunContext _context;
        private readonly IHubContext<CoordinatesHub> _hubContext;

        public CoordinateService(BikeAndRunContext context, IHubContext<CoordinatesHub> hubContext)
        {
            _context = context;
            _hubContext = hubContext;
        }

        /// <summary>
        /// Adds received coordiantes from a mobile device to the database.
        /// </summary>
        /// <param name="coordinatesFromDeviceVM">structure to decode the coordinates from.</param>
        /// <exception cref="Exception">thrown if a condition is not valid that we want to be valid.</exception>
        public void AddCordinates(CoordinatesFromDeviceVM coordinatesFromDeviceVM)
        {
            // finds participant with InviteKey
            var participant = _context.Participants.FirstOrDefault(p => p.InviteKey == coordinatesFromDeviceVM.InviteKey);
            if (participant == null)
            {
                throw new Exception("Ikke gyldig invitasjonsnøkkel");
            }

            var relayStage = _context.RelayStages.FirstOrDefault(rs => rs.Id == participant.RelayStageId);
            var relay = _context.Relays.FirstOrDefault(r => r.Id == relayStage.RelayId);

            // If relay has not started, do nothing with the coordinate
            if (!relay.HasStarted)
            {
                return;
            }

            DateTime dateNow = DateTime.Now;
            var newCoordinate = new Coordinate()
            {
                Latitude = coordinatesFromDeviceVM.Latitude,
                Longitude = coordinatesFromDeviceVM.Longitude,
                ParticipantId = participant.Id,
                Timestamp = dateNow,
            };
            _context.Coordinates.Add(newCoordinate);

            // if not the first stage and not in racing stage
            if (relayStage.StageNumber > 1 && !participant.IsInRacingStage)
            {
                var team = _context.Teams.FirstOrDefault(t => t.Id == participant.TeamId);
                var allRelayStages = _context.RelayStages.Where(rs => rs.RelayId == relay.Id).ToList();
                var prevRelayStage = allRelayStages.Find(rs => rs.StageNumber == relayStage.StageNumber - 1);
                var prevParticipant = _context.Participants.FirstOrDefault(p => p.TeamId == team.Id && p.RelayStageId == prevRelayStage.Id);
                // if participant in prev stage number is in race stage.
                if (prevParticipant.IsInRacingStage)
                {
                    var distance = CoordinateUtils.SimpleDistanceBetween(newCoordinate.Latitude, newCoordinate.Longitude, relay.Latitude, relay.Longitude);
                    double radius = relay.ExchangeRadiusMeters / 110000;
                    // if outside of radius
                    if (distance > radius)
                    {
                        var participantLastCoordinates = GetLastCoordinates(participant.Id, relay.Id);
                        var prevParticipantLastCoordinates = GetLastCoordinates(prevParticipant.Id, relay.Id);
                        // if participant and prev participant has two or more total coordinates registred
                        if (participantLastCoordinates.Count >= 2 && prevParticipantLastCoordinates.Count >= 2)
                        {
                            // if last two coordinates is inside the radius, and the last two coordinates of pref participant is inside the radius (this participant gets in racing, the prev not)
                            if (IsAllCoordinatesOutsideRadius(participantLastCoordinates, relay, radius) && IsAllCoordinatesInsideRadius(prevParticipantLastCoordinates, relay, radius))
                            {
                                // register relay stage change
                                participant.IsInRacingStage = true;
                                prevParticipant.IsInRacingStage = false;
                                _context.SaveChanges();
                                return;
                            }

                            var minutesSincePrevPLastCoord = (dateNow - prevParticipantLastCoordinates[0].Timestamp).TotalMinutes;
                            // if over 10 minutes since prev participant har send coordinates (phone is dead for an example)
                            if (minutesSincePrevPLastCoord >= 10)
                            {
                                // register relay stage change
                                participant.IsInRacingStage = true;
                                prevParticipant.IsInRacingStage = false;
                                _context.SaveChanges();
                                return;
                            }
                        }

                        var minutesSinceRelayStart = (dateNow - relay.StartTime).TotalMinutes;
                        // one or less coordinate from prev particpant and the relay has been active for atleast 15 minutes
                        if (prevParticipantLastCoordinates.Count < 2 && minutesSinceRelayStart > 15)
                        {
                            // register relay stage change on team
                            participant.IsInRacingStage = true;
                            prevParticipant.IsInRacingStage = false;
                            _context.SaveChanges();
                            return;
                        }
                    }
                }
            }
            _context.SaveChanges();
        }

        /// <summary>
        /// Get the last two coordiantes on a user.
        /// </summary>
        /// <param name="participantId">User to fetch data from</param>
        /// <param name="relayId">Relay to find the user from</param>
        /// <returns>Two coordinate pairs</returns>
        public List<CoordinateDTO> GetLastCoordinates(long participantId, long relayId)
        {
            var coordinates = (from coordinate in _context.Coordinates
                               join participant in _context.Participants on coordinate.ParticipantId equals participant.Id
                               join team in _context.Teams on participant.TeamId equals team.Id
                               join relayStage in _context.RelayStages on participant.RelayStageId equals relayStage.Id
                               where participant.Id.Equals(participantId)
                               orderby coordinate.Timestamp descending
                               select new
                               {
                                   Id = coordinate.Id,
                                   Latitude = coordinate.Latitude,
                                   Longitude = coordinate.Longitude,
                                   Timestamp = coordinate.Timestamp,
                                   ParticipantId = participant.Id,
                               }).ToList().Take(2);


            var coordinatesDTO = coordinates.Select(c => new CoordinateDTO()
            {
                Id = c.Id,
                Latitude = c.Latitude,
                Longitude = c.Longitude,
                Timestamp = c.Timestamp,
                ParticipantId = c.ParticipantId,
            }).ToList();
            return coordinatesDTO;
        }

        /// <summary>
        /// Checks whether coordiantes as defined by us, are within the radius defined for the relay.
        /// </summary>
        /// <param name="coordinates">Data to check against</param>
        /// <param name="relay">Relay to fetch data from</param>
        /// <param name="radius">Condition to test coordinates against</param>
        /// <returns>True is it's inside the radius</returns>
        public bool IsAllCoordinatesInsideRadius(List<CoordinateDTO> coordinates, Relay relay, double radius)
        {
            foreach (var coordinate in coordinates)
            {
                var distance = CoordinateUtils.SimpleDistanceBetween(coordinate.Latitude, coordinate.Longitude, relay.Latitude, relay.Longitude);
                if (distance > radius) // if outside radius
                {
                    return false;
                }
            }
            return true;
        }

        /// <summary>
        /// Used to check whether a participant is outside the radius, used in relay stage swapping.
        /// </summary>
        /// <param name="coordinates">Data to fetch coordinates from</param>
        /// <param name="relay">Relay to fetch data against</param>
        /// <param name="radius">Condition to check against a coordinate value</param>
        /// <returns>True if it's outside the radius</returns>
        public bool IsAllCoordinatesOutsideRadius(List<CoordinateDTO> coordinates, Relay relay, double radius)
        {
            foreach (var coordinate in coordinates)
            {
                var distance = CoordinateUtils.SimpleDistanceBetween(coordinate.Latitude, coordinate.Longitude, relay.Latitude, relay.Longitude);
                if (distance < radius) // if inside radius
                {
                    return false;
                }
            }
            return true;
        }
    }
}
