﻿using BikeAndRunAPI.Data;
using BikeAndRunAPI.Data.DTO;
using BikeAndRunAPI.Data.Models;
using BikeAndRunAPI.Jobs;
using BikeAndRunAPI.Scripts;
using BikeAndRunAPI.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;

namespace BikeAndRunAPI.Services
{
    public class RelayService
    {
        private readonly BikeAndRunContext _context;
        private readonly PushRoutine _pushRoutine;

        public RelayService(BikeAndRunContext context, PushRoutine pushRoutine)
        {
            _context = context;
            _pushRoutine = pushRoutine;
        }

        /// <summary>
        /// Starts a websocket and starts piping data to front end.
        /// </summary>
        /// <param name="relayId">Relay to start</param>
        /// <exception cref="Exception">Thrown if race is unable to be found</exception>
        public void StartRelay(long relayId)
        {
            var _relay = _context.Relays.FirstOrDefault(r => r.Id == relayId);
            if (_relay == null)
            {
                throw new Exception("Fant ikke løp");
            }
            _relay.HasStarted = true;
            _context.SaveChanges();
            _ = _pushRoutine.StartPushRoutine(_relay.FrontendUpdateSeconds, relayId);
        }


        // this will only run when the API starts up. (in case the API crashes/restartes, it needs to start websockets again)
        public void StartPushRoutineForStartedRelays()
        {
            
            var _relays = _context.Relays.ToList();

            foreach (var relay in _relays)
            {
                if (relay.HasStarted)
                {
                    _ = _pushRoutine.StartPushRoutine(relay.FrontendUpdateSeconds, relay.Id);
                }
            }
        }

        /// <summary>
        /// Fetches a relay by id, used in frontend to display a single relay.
        /// </summary>
        /// <param name="id">Relay to fetch from the database</param>
        /// <returns>Structure with relay data that is user facing</returns>
        /// <exception cref="Exception">Thrown if relay is not present in the database</exception>
        public RelayDetailDTO GetRelayById(long id)
        {
            var relay = _context.Relays.FirstOrDefault(r => r.Id == id);
            if (relay == null)
            {
                throw new Exception("Fant ikke løp");
            }

            var relayStages = _context.RelayStages.Where(rs => rs.RelayId == relay.Id).ToList();
            var participants = _context.Participants.Where(p => relayStages.Select(rs => rs.Id).Contains(p.RelayStageId)).ToList();
            var teams = _context.Teams.Where(t => participants.Select(p => p.TeamId).Contains(t.Id)).ToList();
            var participantsDTO = participants.Select(p => new ParticipantDTO()
            {
                Id = p.Id,
                TeamId = p.TeamId,
                TeamName = teams.Where(t => t.Id == p.TeamId).Select(t => t.TeamName).Single(),
                RelayStageId = p.RelayStageId,
                StageNumber = relayStages.Where(rs => rs.Id == p.RelayStageId).Select(rs => rs.StageNumber).Single(),
                InviteKey = p.InviteKey,
                HasJoined = p.HasJoined,
                IsInRacingStage = p.IsInRacingStage,
            }).ToList();

            var relayStagesDTO = relayStages.Select(rs => new RelayStageDTO()
            {
                StageTypeId = rs.StageTypeId,
                EndLongitude = rs.EndLongitude,
                EndLatitude = rs.EndLatitude,
            }).ToList();



            var relayDetailDTO = new RelayDetailDTO()
            {
                Id = relay.Id,
                Latitude = relay.Latitude,
                Longitude = relay.Longitude,
                Location = relay.Location,
                StartTime = relay.StartTime,
                EndTime = relay.EndTime,
                Information = relay.Information,
                HasStarted = relay.HasStarted,
                Participants = participantsDTO,
                ExchangeRadiusMeters = relay.ExchangeRadiusMeters,
                FrontendUpdateSeconds = relay.FrontendUpdateSeconds,
                RelayStages = relayStagesDTO,
                Teams = teams,
            };

            return relayDetailDTO;
        }

        /// <summary>
        /// Gets every single relay in the database. Used for displaying on front end.
        /// </summary>
        /// <returns>List of relays</returns>
        public List<Relay> GetAllRelays()
        {
            var allRelays = _context.Relays.ToList();
            if (allRelays.Any())
            {
                return allRelays;
            }
            else
            {
                return null;
            }
        }

        public void EditRelayById(NewOrEditRelayVM newOrEditRelayVM, long id)
        {
            var relay = _context.Relays.FirstOrDefault(r => r.Id == id);

            // if relay does not exist
            if (relay == null)
            {
                throw new Exception("Fant ikke løp");
            }
            relay.StartTime = newOrEditRelayVM.StartTime;
            relay.EndTime = newOrEditRelayVM.EndTime;
            relay.Latitude = newOrEditRelayVM.Latitude;
            relay.Longitude = newOrEditRelayVM.Longitude;
            relay.Location = newOrEditRelayVM.Location;
            relay.Information = newOrEditRelayVM.Information;
            relay.ExchangeRadiusMeters = newOrEditRelayVM.ExchangeRadiusMeters;
            relay.FrontendUpdateSeconds = newOrEditRelayVM.FrontendUpdateSeconds;


            var relayStages = _context.RelayStages.Where(rs => rs.RelayId == relay.Id).OrderBy(rs => rs.StageNumber).ToList();
            var participants = _context.Participants.Where(p => relayStages.Select(rs => rs.Id).Contains(p.RelayStageId)).ToList();
            var teams = _context.Teams.Where(t => participants.Select(p => p.TeamId).Contains(t.Id)).ToList();

            // if more relay stages in edited relay
            if (relayStages.Count < newOrEditRelayVM.RelayStages.Count)
            {
                for (int i = 0; i < relayStages.Count; i++)
                {
                    var stageType = _context.StageTypes.FirstOrDefault(st => st.Id == newOrEditRelayVM.RelayStages[i].StageTypeId);
                    relayStages[i].StageType = stageType;
                    relayStages[i].EndLatitude = newOrEditRelayVM.RelayStages[i].Lat;
                    relayStages[i].EndLongitude = newOrEditRelayVM.RelayStages[i].Lng;
                }
                for (int i = relayStages.Count; i < newOrEditRelayVM.RelayStages.Count; i++)
                {
                    var stageType = _context.StageTypes.FirstOrDefault(st => st.Id == newOrEditRelayVM.RelayStages[i].StageTypeId);
                    var newRelayStage = new RelayStage()
                    {
                        Relay = relay,
                        StageType = stageType,
                        StageNumber = i + 1,
                        EndLatitude = newOrEditRelayVM.RelayStages[i].Lat,
                        EndLongitude = newOrEditRelayVM.RelayStages[i].Lng,
                    };
                    _context.RelayStages.Add(newRelayStage);

                    // adds participants to the new relay stage
                    foreach (var team in teams)
                    {
                        if (newOrEditRelayVM.TeamIds.Contains(team.Id))
                        {
                            var newParticipant = new Participant()
                            {
                                Team = team,
                                RelayStage = newRelayStage,
                                InviteKey = Generator.RandomString(6),
                                HasJoined = false,
                                IsInRacingStage = false,
                            };
                            _context.Participants.Add(newParticipant);
                        }
                    }
                }

                // if less relay stages in edited relay
            }
            else if (relayStages.Count > newOrEditRelayVM.RelayStages.Count)
            {
                for (int i = 0; i < newOrEditRelayVM.RelayStages.Count; i++)
                {
                    var stageType = _context.StageTypes.FirstOrDefault(st => st.Id == newOrEditRelayVM.RelayStages[i].StageTypeId);
                    relayStages[i].StageType = stageType;
                    relayStages[i].EndLatitude = newOrEditRelayVM.RelayStages[i].Lat;
                    relayStages[i].EndLongitude = newOrEditRelayVM.RelayStages[i].Lng;
                }
                // remove participants and relay stages
                for (int i = newOrEditRelayVM.RelayStages.Count; i < relayStages.Count; i++)
                {
                    var relayStageToDelete = relayStages[i];
                    var participantsToDelete = _context.Participants.Where(p => p.RelayStageId == relayStageToDelete.Id).ToList();
                    _context.Participants.RemoveRange(participantsToDelete);
                    _context.RelayStages.Remove(relayStageToDelete);
                }

                // if the same amout of relay stages
            }
            else
            {
                for (int i = 0; i < newOrEditRelayVM.RelayStages.Count; i++)
                {
                    var stageType = _context.StageTypes.FirstOrDefault(st => st.Id == newOrEditRelayVM.RelayStages[i].StageTypeId);
                    relayStages[i].StageType = stageType;
                    relayStages[i].EndLatitude = newOrEditRelayVM.RelayStages[i].Lat;
                    relayStages[i].EndLongitude = newOrEditRelayVM.RelayStages[i].Lng;
                }
            }

            // remove team from relay (in practice remove participants from team from relay)
            foreach (Team team in teams)
            {
                if (!newOrEditRelayVM.TeamIds.Contains(team.Id))
                {
                    var participantsToRemove = _context.Participants.Where(p => p.TeamId == team.Id);
                    _context.Participants.RemoveRange(participantsToRemove);
                }
            }

            // add new participant to relay (in practice add participants from team to relay)
            foreach (long teamId in newOrEditRelayVM.TeamIds)
            {
                var teamIndex = teams.FindIndex(t => t.Id == teamId);
                if (teamIndex < 0)
                {
                    var team = _context.Teams.FirstOrDefault(t => t.Id == teamId);
                    for (var i = 0; i < relayStages.Count; i++)
                    {
                        var newParticipant = new Participant()
                        {
                            Team = team,
                            RelayStageId = relayStages[i].Id,
                            InviteKey = Generator.RandomString(6),
                            HasJoined = false,
                            IsInRacingStage = false,
                        };
                        _context.Participants.Add(newParticipant);
                    }


                }
            }

            _context.SaveChanges();
        }

        /// <summary>
        /// Adds a new relay to the database with user inputted data from frontend.
        /// </summary>
        /// <param name="newRelayVM">Contains the new relays data</param>
        /// <returns>Integer ID of new relay</returns>
        public long CreateNewRelay(NewOrEditRelayVM newOrEditRelayVM)
        {

            var newRelay = new Relay()
            {
                StartTime = newOrEditRelayVM.StartTime,
                EndTime = newOrEditRelayVM.EndTime,
                Latitude = newOrEditRelayVM.Latitude,
                Longitude = newOrEditRelayVM.Longitude,
                Location = newOrEditRelayVM.Location,
                Information = newOrEditRelayVM.Information,
                HasStarted = false,
            };
            _context.Relays.Add(newRelay);

            var newRelayStages = new List<RelayStage>();
            for (int i = 0; i < newOrEditRelayVM.RelayStages.Count; i++)
            {
                var stageType = _context.StageTypes.FirstOrDefault(st => st.Id == newOrEditRelayVM.RelayStages[i].StageTypeId);
                var newRelayStage = new RelayStage()
                {
                    Relay = newRelay,
                    StageType = stageType,
                    StageNumber = (i + 1),
                    EndLatitude = newOrEditRelayVM.RelayStages[i].Lat,
                    EndLongitude = newOrEditRelayVM.RelayStages[i].Lng,
                };
                _context.RelayStages.Add(newRelayStage);
                newRelayStages.Add(newRelayStage);
            };

            foreach (var teamId in newOrEditRelayVM.TeamIds)
            {
                var team = _context.Teams.FirstOrDefault(t => t.Id == teamId);
                for (int i = 0; i < newRelayStages.Count; i++)
                {
                    var newParticpant = new Participant()
                    {
                        RelayStage = newRelayStages[i],
                        Team = team,
                        InviteKey = Generator.RandomString(6),
                        HasJoined = false,
                        IsInRacingStage = i == 0 ? true : false,
                    };
                    _context.Participants.Add(newParticpant);
                };
            };

            _context.SaveChanges();
            return newRelay.Id;
        }

        /// <summary>
        /// Removes a single relay from database by id.
        /// </summary>
        /// <param name="relayId">Relay to remove</param>
        /// <exception cref="Exception">Thrown if the id is not in a valid range</exception>
        public void DeleteRelayByRelayId(long relayId)
        {
            var relay = _context.Relays.FirstOrDefault(r => r.Id == relayId);
            if (relay == null)
            {
                throw new Exception("Fant ikke løp");
            }

            var relayStages = _context.RelayStages.Where(rs => rs.RelayId == relayId).ToList();
            var participants = _context.Participants.Where(p => relayStages.Select(rs => rs.Id).Contains(p.RelayStageId)).ToList();
            var coordinates = _context.Coordinates.Where(c => participants.Select(p => p.Id).Contains(c.Participant.Id)).ToList();

            _context.Coordinates.RemoveRange(coordinates);
            _context.Participants.RemoveRange(participants);
            _context.RelayStages.RemoveRange(relayStages);
            _context.Relays.Remove(relay);
            _context.SaveChanges();

        }
    }
}
