﻿using BikeAndRunAPI.Data;
using BikeAndRunAPI.Data.Models;
using BikeAndRunAPI.ViewModels;
using Microsoft.AspNetCore.JsonPatch;
using System;
using System.Collections.Generic;
using System.Linq;

namespace BikeAndRunAPI.Services
{
    public class TeamService
    {
        private readonly BikeAndRunContext _context;
        public TeamService(BikeAndRunContext context)
        {
            _context = context;
        }

        /// <summary>
        /// Gets all the teams in the database, used for displaying list-wise on front end.
        /// </summary>
        /// <returns>List of teams</returns>
        public List<Team> GetAllTeams()
        {
            var allTeams = _context.Teams.ToList();
            return allTeams;
        }

        /// <summary>
        /// Adds new team with new user data input.
        /// </summary>
        /// <param name="team">New team data</param>
        public void AddTeam(TeamVM team)
        {
            var teamToAdd = new Team()
            {
                TeamName = team.TeamName,
                Email = team.Email
            };
            _context.Teams.Add(teamToAdd);
            _context.SaveChanges();
        }

        /// <summary>
        /// Removes a single team from database by id, used by admins to remove teams.
        /// </summary>
        /// <param name="id">Id of team to remove</param>
        /// <exception cref="Exception">thrown if team has participant or if team could not be found with the id</exception>
        public void DeleteTeam(long id)
        {
            var teamToDelete = _context.Teams.FirstOrDefault(t => t.Id == id);
            if (teamToDelete == null)
            {
                var errorMessage = "Fant ingen team med id: {0}" + id;
                throw new Exception(errorMessage);
            }
           
            var participant = _context.Participants.FirstOrDefault(t => t.TeamId == teamToDelete.Id);
            if (participant != null)
            {
                throw new Exception("Kan ikke slette team som har participant koblet til teamet.");
            }

            _context.Teams.Remove(teamToDelete);
            _context.SaveChanges();
        }

        /// <summary>
        /// Updates a team with new values for both fields.
        /// </summary>
        /// <param name="id">Id of team to update</param>
        /// <param name="teamVM">New data to update with</param>
        public void EditTeam(long id, TeamVM teamVM)
        {
            var teamToUpdate = _context.Teams.FirstOrDefault(t => t.Id == id);

            teamToUpdate.TeamName = teamVM.TeamName;
            teamToUpdate.Email = teamVM.Email;

            _context.SaveChanges();
        }

        /// <summary>
        /// Updates a single field of the two field structure of a team.
        /// </summary>
        /// <param name="id">Id of team to update</param>
        /// <param name="team">What field to update and what should the new value should be.</param>
        public void EditTeamPatch(long id, JsonPatchDocument team)
        {
            var teamToUpdate = _context.Teams.FirstOrDefault(t => t.Id == id);
            if (teamToUpdate != null)
            {
                team.ApplyTo(teamToUpdate);
            }
            _context.SaveChanges();
        }
    }
}