using BikeAndRunAPI.Data;
using BikeAndRunAPI.Hubs;
using BikeAndRunAPI.Jobs;
using BikeAndRunAPI.Services;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.IdentityModel.Tokens;
using Microsoft.OpenApi.Models;


namespace BikeAndRunAPI
{
    public class Startup
    {

        private string ConnectionString { get; set; }
        public string KeycloakServerRealm { get; set; }
        public string KeycloakMetadata { get; set; }
        public string KeycloakClientId { get; set; }
        public string KeycloakClientSecret { get; set; }

        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
            ConnectionString = Configuration.GetConnectionString("DefaultConnectionString");
            KeycloakServerRealm = Configuration.GetSection("Keycloak")["ServerRealm"];
            KeycloakMetadata = Configuration.GetSection("Keycloak")["ServerRealm"];
            KeycloakClientId = Configuration.GetSection("Keycloak")["ClientId"];
            KeycloakClientSecret = Configuration.GetSection("Keycloak")["ClientSecret"];
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {

            services.AddDbContext<BikeAndRunContext>(opt => opt.UseSqlServer(ConnectionString));
            services.AddControllers().AddNewtonsoftJson();
            services.AddSignalR();
            services.AddTransient<CoordinateService>();
            services.AddScoped<PushRoutine>();
            services.AddScoped<StartAndEndRaceRoutine>();
            services.AddTransient<TeamService>();
            services.AddTransient<RelayService>();
            services.AddTransient<ParticipantService>();
            services.AddTransient<StageTypeService>();
            services.AddMvc();

            // oauth2 with keycloak server

            services.AddAuthentication(options =>
            {
                options.DefaultScheme = JwtBearerDefaults.AuthenticationScheme;
                options.DefaultAuthenticateScheme = JwtBearerDefaults.AuthenticationScheme;
                options.DefaultChallengeScheme = JwtBearerDefaults.AuthenticationScheme;
            }).AddJwtBearer(options =>
            {
                options.Authority = KeycloakServerRealm;
                options.Audience = KeycloakClientId;
                options.TokenValidationParameters = new TokenValidationParameters
                {
                    ValidateAudience = false,
                    ValidateIssuer = false,
                    ValidIssuer = KeycloakServerRealm,
                    ValidateLifetime = false,
                    RequireExpirationTime = false
                };

            });
            //services.AddAuthorization();

            // CORS
            services.AddCors(options => options.AddPolicy("CorsPolicy",
                    builder =>
                    {
                        builder.AllowAnyHeader()
                               .AllowAnyMethod()
                               .SetIsOriginAllowed((host) => true)
                               .AllowCredentials();
                    }));

            services.AddSwaggerGen(c =>
            {
                c.SwaggerDoc("v1", new OpenApiInfo { Title = "BikeAndRunAPI", Version = "v1" });
            });
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            app.UseCors("CorsPolicy");

            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
                app.UseSwagger();
                app.UseSwaggerUI(c => c.SwaggerEndpoint("/swagger/v1/swagger.json", "BikeAndRunAPI v1"));
            }

            app.UseHttpsRedirection();
            app.UseRouting();
            //app.UseAuthentication();
            //app.UseAuthorization();
            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
                endpoints.MapHub<CoordinatesHub>("/coordinates");
            });

            // remove the comment for seeding dummy data to database
            // DbInitializer.Seed(app);
        }

    }
}
