﻿using BikeAndRunAPI.Controllers;
using BikeAndRunAPI.Data;
using BikeAndRunAPI.Data.Models;
using BikeAndRunAPI.Jobs;
using BikeAndRunAPI.Services;
using BikeAndRunAPI.ViewModels;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Data.Sqlite;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Logging;
using Moq;
using System;
using System.Collections.Generic;
using System.Data.Common;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Xunit;
using Xunit.Abstractions;

namespace IntegrationTests
{
    public class RelayIntegrationTests
    {
        private readonly DbConnection connection;
        private readonly DbContextOptions<BikeAndRunContext> dbContextOptions;
        private RelayService relayService;
        private readonly PushRoutine pushRoutine;
        public StartAndEndRaceRoutine startAndEndRaceRoutine;

        private readonly Mock<ILogger<CoordinateController>> logger = null; // we do not use this for tests but we have to create it as controllers demand it.

        private readonly ITestOutputHelper output; // use the method output.Writeline to print state in tests.

        public RelayIntegrationTests(ITestOutputHelper output)
        {
            connection = new SqliteConnection("Filename=:memory:");
            connection.Open();

            logger = new Mock<ILogger<CoordinateController>>();
            logger.Setup(x => x.Log<It.IsAnyType>(
                It.IsAny<LogLevel>(), It.IsAny<EventId>(), It.IsAny<It.IsAnyType>(), It.IsAny<Exception>(), (Func<It.IsAnyType, Exception, string>)It.IsAny<object>()));

            dbContextOptions = new DbContextOptionsBuilder<BikeAndRunContext>().UseSqlite(connection).Options;

            using var ctx = new BikeAndRunContext(dbContextOptions);

            ctx.Database.EnsureCreated();

            ctx.AddRange(
                new Relay { 
                    Id = 1,
                    StartTime = new DateTime(2012,12,12), 
                    EndTime = new DateTime(2012,12,13),
                    Latitude = 60.79574,
                    Longitude = 10.69155,
                    Location = "Gjøvik",
                    Information = "Ta med ekstra sokker og et sett med skift",
                    ExchangeRadiusMeters = 30,
                    FrontendUpdateSeconds = 3,
                    HasStarted = false
                });
            ctx.SaveChanges();

            this.output = output;
        }

        private BikeAndRunContext CreateContext() => new(dbContextOptions);

        internal void DisposeContext() => connection.Dispose();

        [Fact]
        public void TestRelayGet()
        {
            using var ctx = CreateContext();
            relayService = new RelayService(ctx, pushRoutine);

            var controller = new RelayController(logger.Object, pushRoutine, relayService, startAndEndRaceRoutine);

            var relays = controller.GetAllRelays();

            var okResult = relays as OkObjectResult;
            Assert.NotNull(okResult);
            Assert.Equal(200, okResult.StatusCode);

            var relay = okResult.Value as List<Relay>;

            Assert.NotNull(relay);
            Assert.Single(relay);
            Assert.Collection(relay, item => Assert.Contains("Ta med ekstra sokker og et sett med skift", item.Information));

            DisposeContext();
        }

        [Fact]
        public void TestGetRelayWithId()
        {
            using var ctx = CreateContext();
            relayService = new RelayService(ctx, pushRoutine);

            var controller = new RelayController(logger.Object, pushRoutine, relayService, startAndEndRaceRoutine);

            var id = 1;

            var relays = controller.GetRelayById(id);

            var okResult = relays as OkObjectResult;
            Assert.NotNull(okResult);
            Assert.Equal(200, okResult.StatusCode);

            DisposeContext();
        }


        [Fact]
        public void TestRelayWithInvalidId()
        {
            using var ctx = CreateContext();
            relayService = new RelayService(ctx, pushRoutine);

            var controller = new RelayController(logger.Object, pushRoutine, relayService, startAndEndRaceRoutine);

            var id = 1;

            var relays = controller.GetRelayById(-999);

            var notFoundRes = relays as NotFoundObjectResult;
            Assert.Equal(404, notFoundRes.StatusCode);

            DisposeContext();
        }

        [Fact]
        public void TestNewRelayCreation()
        {
            using var ctx = CreateContext();
            relayService = new RelayService(ctx, pushRoutine);

            var controller = new RelayController(logger.Object, pushRoutine, relayService, startAndEndRaceRoutine);

            // we are not interested in this data so we don't have to initialize "real" data.
            var newTeamIds = new List<long>() { };
            var newRelayStages = new List<RelayStageFromClient> { };

            NewOrEditRelayVM newOrEditRelayVm = new NewOrEditRelayVM()
            {
                StartTime = new DateTime(2013, 11, 13),
                EndTime = new DateTime(2013, 11, 14),
                Latitude = 57.79574,
                Longitude = 11.69155,
                Location = "Sunnmøre",
                Information = "ta med deg badetøy",
                ExchangeRadiusMeters = 100,
                FrontendUpdateSeconds = 3,
                TeamIds = newTeamIds,
                RelayStages = newRelayStages
            };

            var relays = controller.CreateNewRelay(newOrEditRelayVm); // returns the ID of the new relay.

            var okResult = relays as OkObjectResult;
            Assert.Equal(200, okResult.StatusCode);

            int id = Int32.Parse(okResult.Value.ToString());

            Assert.Equal(2, id);

            DisposeContext();
        }

        [Fact]
        public void TestDeleteRelayById()
        {
            using var ctx = CreateContext();
            relayService = new RelayService(ctx, pushRoutine);

            var controller = new RelayController(logger.Object, pushRoutine, relayService, startAndEndRaceRoutine);

            var id = 1;

            var relays = controller.DeleteRelayById(id); // returns the ID of the new relay.

            var okResult = relays as OkResult;
            Assert.Equal(200, okResult.StatusCode);

            var relayCount = ctx.Relays.Count(t => t.Id == id);

            Assert.Equal(0, relayCount);

            DisposeContext();
        }

        [Fact]
        public void TestEditingExistingRelay()
        {
            using var ctx = CreateContext();
            relayService = new RelayService(ctx, pushRoutine);

            var controller = new RelayController(logger.Object, pushRoutine, relayService, startAndEndRaceRoutine);

            RelayStageFromClient newRelayStageLocation = new RelayStageFromClient()
            {
                StageTypeId = 2,
                Lat = 111,
                Lng = 111,
            };

            var newTeamIds = new List<long>() { 1,2,3 };
            var newRelayStages = new List<RelayStageFromClient> { newRelayStageLocation };

            NewOrEditRelayVM newOrEditRelayVm = new NewOrEditRelayVM()
            {
                Latitude = 57.79574,
                Longitude = 11.69155,
                Location = "Stavanger",
                StartTime = new DateTime(2013, 11, 13),
                EndTime = new DateTime(2013, 11, 14),
                Information = "Ta med Solkrem",
                ExchangeRadiusMeters = 100,
                FrontendUpdateSeconds = 3,
                TeamIds = newTeamIds,
                RelayStages = newRelayStages
            };

            var relays = controller.EditRelay(newOrEditRelayVm, 1); // returns the ID of the new relay.

            var editedRelay = ctx.Relays.FirstOrDefault(t => t.Id == 1);

            Assert.NotNull(editedRelay);
            Assert.Equal("Stavanger", editedRelay.Location);
            Assert.Equal("Ta med Solkrem", editedRelay.Information);

            DisposeContext();
        }
    }
}
