using BikeAndRunAPI.Controllers;
using BikeAndRunAPI.Data;
using BikeAndRunAPI.Data.Models;
using BikeAndRunAPI.Services;
using BikeAndRunAPI.ViewModels;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Data.Sqlite;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Logging;
using Moq;
using System;
using System.Collections.Generic;
using System.Data.Common;
using System.Linq;
using Xunit;
using Xunit.Abstractions;


namespace UnitTests.TeamTests
{
    public class TeamIntegrationTests
    {
        private readonly DbConnection connection;
        private readonly DbContextOptions<BikeAndRunContext> dbContextOptions;
        private TeamService teamService;

        private readonly Mock<ILogger<CoordinateController>> logger = null; // we do not use this for tests but we have to create it as controllers demand it.
        
        private readonly ITestOutputHelper output; // use the method output.Writeline to print state in tests.

        public TeamIntegrationTests(ITestOutputHelper output)
        {
            connection = new SqliteConnection("Filename=:memory:");
            connection.Open();

            logger = new Mock<ILogger<CoordinateController>>();
            logger.Setup(x => x.Log<It.IsAnyType>(
                It.IsAny<LogLevel>(), It.IsAny<EventId>(), It.IsAny<It.IsAnyType>(), It.IsAny<Exception>(), (Func<It.IsAnyType, Exception, string>)It.IsAny<object>()));

            dbContextOptions = new DbContextOptionsBuilder<BikeAndRunContext>().UseSqlite(connection).Options;

            using var ctx = new BikeAndRunContext(dbContextOptions);

            ctx.Database.EnsureCreated();

            ctx.AddRange(
                new Team {TeamName = "Redalen Auto", Email = "RedalenAuto@gmail.com" });
            ctx.SaveChanges();

            this.output = output;
        }

        private BikeAndRunContext CreateContext() => new(dbContextOptions);

        internal void DisposeContext() => connection.Dispose();

        [Fact]
        public void VerifyAmountOfTeams()
        {
            using var ctx = CreateContext();
            teamService = new TeamService(ctx);

            var controller = new TeamController(logger.Object, teamService);

            var teams = controller.GetAllTeams();

            var okResult = teams as OkObjectResult;

            var teamList = okResult.Value as List<Team>;

            Assert.Collection(teamList, item => Assert.Contains("Redalen Auto", item.TeamName));

            DisposeContext();
        }

        [Fact]
        public void AddAndVerifyTeam()
        {
            using var ctx = CreateContext();
            teamService = new TeamService(ctx);

            var controller = new TeamController(logger.Object, teamService);

            var teamToAdd = new TeamVM()
            {
                TeamName = "Srbyen",
                Email = "Srbyen@gmail.com"
            };

            var newTeamResult = controller.AddTeam(teamToAdd);

            var teams = controller.GetAllTeams();

            var okResult = teams as OkObjectResult;

            var teamList = okResult.Value as List<Team>;

            Assert.Equal(2, teamList.Count);

            DisposeContext();
        }

        [Fact]
        public void DeletingNonExistentTeam()
        {
            using var ctx = CreateContext();
            teamService = new TeamService(ctx);
            var controller = new TeamController(logger.Object, teamService);

            var errorTeam = controller.DeleteTeam(-999);

            var notFoundRes = errorTeam as NotFoundObjectResult;

            Assert.Equal(404, notFoundRes.StatusCode);

            DisposeContext();
        }

        [Fact]
        public void EditSingleTeam()
        {
            using var ctx = CreateContext();
            teamService = new TeamService(ctx);
            var controller = new TeamController(logger.Object, teamService);

            var newTeam = new TeamVM()
            {
                TeamName = "NewTeamName",
                Email = "NewEmail@gmail.com"
            };

            var id = 1;

            var editedTeam = controller.EditTeam(id, newTeam);

            var editedTeamResult = editedTeam as OkResult;
            Assert.Equal(200, editedTeamResult.StatusCode);

            var editedTeamFromDatabase = ctx.Teams.FirstOrDefault(t => t.Id == 1);

            Assert.NotNull(editedTeamFromDatabase);
            Assert.Equal("NewTeamName", editedTeamFromDatabase.TeamName);
            Assert.Equal("NewEmail@gmail.com", editedTeamFromDatabase.Email);

            DisposeContext();
        }
    }
}