using BikeAndRunAPI.Data;
using BikeAndRunAPI.Data.DTO;
using BikeAndRunAPI.Data.Models;
using BikeAndRunAPI.Hubs;
using BikeAndRunAPI.Jobs;
using BikeAndRunAPI.Scripts;
using BikeAndRunAPI.Services;
using Microsoft.AspNetCore.SignalR;
using Microsoft.Data.Sqlite;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.DependencyInjection;
using System;
using System.Collections.Generic;
using System.Data.Common;
using System.Threading.Tasks;
using Xunit;
using Xunit.Abstractions;

namespace UnitTests
{
    public class UnitTest
    {
        private readonly DbConnection connection;
        private readonly DbContextOptions<BikeAndRunContext> dbContextOptions;
        private CoordinateService coordinateService;
       
        private readonly RelayService relayService;

        private readonly IServiceScopeFactory scopeFactory;
        private readonly IHubContext<CoordinatesHub> hubContext;

        private PushRoutine pushRoutine;

        private readonly ITestOutputHelper output; // use this variable .writeline to be able to see any print statements.

        public UnitTest(ITestOutputHelper output)
        {
            connection = new SqliteConnection("Filename=:memory:");
            connection.Open();

            dbContextOptions = new DbContextOptionsBuilder<BikeAndRunContext>().UseSqlite(connection).Options;

            var ctx = new BikeAndRunContext(dbContextOptions);

            ctx.Database.EnsureCreated();
            ctx.AddRange(new Relay
            {
                StartTime = DateTime.Now.AddDays(10),
                EndTime = DateTime.Now.AddDays(10).AddHours(2),
                Latitude = 60.796741,
                Longitude = 10.694083,
                Location = "Gjvik stasjon",
                Information = "Det er meldt regn s ta med ekstra skift.",
                HasStarted = false,
            });

            ctx.SaveChanges();

            pushRoutine = new PushRoutine(scopeFactory, hubContext);
            
            relayService = new RelayService(ctx, pushRoutine);

            long id = 1;

            var relay = ctx.Relays.Find(id);

            output.WriteLine(relay.StartTime.ToString());
            output.WriteLine(relay.EndTime.ToString());

            this.output = output;
        }

        private BikeAndRunContext CreateContext() => new(dbContextOptions);

        internal void DisposeContext() => connection.Dispose();

        [Fact]
        public void CheckIfCoordinateIsInRadius()
        {
            using var ctx = CreateContext();
            double radius = 0.002121320343559723;
            coordinateService = new CoordinateService(ctx, hubContext);

            Assert.NotNull(coordinateService);

            var relay = new Relay
            {
                StartTime = DateTime.Now.AddDays(10),
                EndTime = DateTime.Now.AddDays(10).AddHours(2),
                Latitude = 60.796741,
                Longitude = 10.694083,
                Location = "Gjvik stasjon",
                Information = "Det er meldt regn s ta med ekstra skift.",
                HasStarted = false,
            };

            var coordinateDTO = new List<CoordinateDTO>
            {
               new CoordinateDTO { Latitude=60.796741 + 0.00005, Longitude=10.694083 + 0.000001, ParticipantId=1, TeamName="Headit AS",  Timestamp = new DateTime()},
               new CoordinateDTO { Latitude=60.796741 + 0.00004, Longitude=10.694083 + 0.000002, ParticipantId=2, TeamName="Headit AS",  Timestamp = new DateTime()},
               new CoordinateDTO { Latitude=60.796741 + 0.00003, Longitude=10.694083 + 0.000003, ParticipantId=3, TeamName="Headit AS",  Timestamp = new DateTime()},
            };

            bool res = coordinateService.IsAllCoordinatesInsideRadius(coordinateDTO, relay, radius);

            Assert.True(res);

            DisposeContext();
        }


        [Fact]
        public void CheckIfCoordinateIsOutsideRadius()
        {
            using var ctx = CreateContext();
            double radius = 0.002121320343559723;
            coordinateService = new CoordinateService(ctx, hubContext);

            Assert.NotNull(coordinateService);

            var relay = new Relay
            {
                StartTime = DateTime.Now,
                EndTime = DateTime.Now.AddSeconds(15.0),
                Latitude = 60.796741,
                Longitude = 10.694083,
                Location = "Gjvik stasjon",
                Information = "Det er meldt regn s ta med ekstra skift.",
                HasStarted = false,
            };

            var coordinateDTO = new List<CoordinateDTO>
            {
               new CoordinateDTO { Latitude=50.0, Longitude=50.0, ParticipantId=1, TeamName="Headit AS",  Timestamp = new DateTime()},
               new CoordinateDTO { Latitude=51.0, Longitude=49.0, ParticipantId=2, TeamName="Headit AS",  Timestamp = new DateTime()},
               new CoordinateDTO { Latitude=52.0, Longitude=48.0, ParticipantId=3, TeamName="Headit AS",  Timestamp = new DateTime()},
            };

            bool res = coordinateService.IsAllCoordinatesOutsideRadius(coordinateDTO, relay, radius);

            Assert.True(res);

            DisposeContext();
        }

        [Fact]
        public void TestVectorDistance()
        {
            using var ctx = CreateContext();

            double lat1 = 50.001;
            double lat2 = 50.002;

            double lat3 = 50.008;
            double lat4 = 50.009;

            var distanceVal = CoordinateUtils.SimpleDistanceBetween(lat1, lat2, lat3, lat4);

            var roundedDistanceValue = Math.Round(distanceVal, 8);

            double desiredResult = 0.00989949;

            Assert.Equal(desiredResult, roundedDistanceValue);

            DisposeContext();
        }

        [Fact]
        public void TestHaversineFormula()
        {
            using var ctx = CreateContext();

            double lat1 = 51.01;
            double lat2 = 52.01;

            double lat3 = 53.08;
            double lat4 = 53.09;

            var distanceVal = CoordinateUtils.Haversine(lat1, lat2, lat3, lat4);

            var roundedDistanceValue = Math.Round(distanceVal, 3);

            double desiredResult = 10713.268;

            Assert.Equal(desiredResult, roundedDistanceValue);

            DisposeContext();
        }
    }
}
