import React, { useState, useEffect } from "react";
import {
  GoogleMap,
  useJsApiLoader,
  Circle,
  Marker,
} from "@react-google-maps/api";
import Button from "@mui/material/Button";
import FormControlLabel from "@mui/material/FormControlLabel";
import Switch from "@mui/material/Switch";

import { sendCoordinates } from "../../utils/apiRequest";
import {
  isAtTarget,
  getDirection,
  getVelocity,
  randomPositionInRadius,
} from "../../utils/misc";

const containerStyle = {
  width: "500px",
  height: "500px",
};

const defaultZoom = 14;

const relay1GPS = {
  lat: 60.794744,
  lng: 11.067623,
};

const radiusMeter = 190;
const radius = 190 / 113000;

const startPoint = relay1GPS;

const lapPoints = [
  {
    ...startPoint,
  },
  {
    lat: startPoint.lat + 0.01,
    lng: startPoint.lng + 0.01,
  },
  {
    lat: startPoint.lat + 0.01,
    lng: startPoint.lng - 0.01,
  },
  {
    ...startPoint,
  },
];

const MovingPlayers = ({ googleKey, participants }) => {
  const { isLoaded } = useJsApiLoader({
    id: "google-map-script",
    googleMapsApiKey: googleKey,
  });

  const [teams, setTeams] = useState([]);
  const [sendGpsSignal, setSendGpsSignal] = useState(true);

  useEffect(() => {
    let teamsPlaceholder = [];

    participants.map((participant) => {
      let index = teamsPlaceholder.findIndex((team) => {
        return team.teamName === participant.teamName;
      });
      if (index === -1) {
        let emptyTeam = {
          teamName: participant.teamName,
          playerRacerIndex: 0,
          players: [],
        };
        teamsPlaceholder.push(emptyTeam);
      }
      index = teamsPlaceholder.findIndex((team) => {
        return team.teamName === participant.teamName;
      });

      let player = {
        participantId: participant.id,
        inviteKey: participant.inviteKey,
        position: startPoint,
        stageNumber: participant.stageNumber,
        nextPathIndex: 1,
        velocity: { lat: 0, lng: 0 },
        targetLocation: participant.isInRacingStage
          ? lapPoints[1]
          : randomPositionInRadius(startPoint, radius),
        inRace: participant.isInRacingStage,
        moving: true,
        speed: generateRandomSpeed(),
        inviteKey: participant.inviteKey,
      };
      teamsPlaceholder[index].players.push(player);
    });

    for (let i = 0; i < teamsPlaceholder.length; i++) {
      teamsPlaceholder[i].players.sort((a, b) => {
        if (a.stageNumber > b.stageNumber) {
          return 1;
        } else if (a.stageNumber < b.stageNumber) {
          return -1;
        } else {
          return 0;
        }
      });
    }
    setTeams(teamsPlaceholder);
  }, []);

  const generateRandomSpeed = () => {
    var randomnum =
      Math.floor(Math.random() * (100000 - 1000) + 1000) / 1000000000;

    randomnum *= 0.2;
    return 0.000082 + randomnum;
  };

  const sendGpsPositions = () => {
    if (sendGpsSignal) {
      teams.map((team) => {
        team.players.map((player) => {
          sendCoordinates(
            player.inviteKey,
            player.position.lat,
            player.position.lng
          );
        });
      });
    }
    setTimeout(() => {
      sendGpsPositions();
    }, 5000);
  };

  const buttonClick = () => {
    sendGpsPositions();
    continuesMovement();
  };

  const continuesMovement = () => {
    setTimeout(() => {
      moveAllPlayers();
      continuesMovement();
    }, 100);
  };

  const moveAllPlayers = () => {
    let teamsPlaceholder = [...teams];

    for (let i = 0; i < teamsPlaceholder.length; i++) {
      for (let j = 0; j < teamsPlaceholder[i].players.length; j++) {
        let playerPlaceholder = { ...teamsPlaceholder[i].players[j] };
        if (playerPlaceholder.moving) {
          let direction = getDirection(
            playerPlaceholder.position,
            playerPlaceholder.targetLocation
          );
          let velocity = getVelocity(direction, playerPlaceholder.speed);
          let newPosition = getNewPosition(
            playerPlaceholder.position,
            velocity
          );

          if (
            isAtTarget(velocity, newPosition, playerPlaceholder.targetLocation)
          ) {
            if (playerPlaceholder.inRace) {
              playerPlaceholder.nextPathIndex++;
              if (playerPlaceholder.nextPathIndex >= lapPoints.length) {
                playerPlaceholder.nextPathIndex = 1;
                playerPlaceholder.inRace = false;
                teamsPlaceholder[i].playerRacerIndex++;
                if (
                  teamsPlaceholder[i].playerRacerIndex <
                  teamsPlaceholder[i].players.length
                ) {
                  teamsPlaceholder[i].players[
                    teamsPlaceholder[i].playerRacerIndex
                  ].inRace = true;
                  teamsPlaceholder[i].players[
                    teamsPlaceholder[i].playerRacerIndex
                  ].targetLocation = lapPoints[1];
                }
              } else {
                playerPlaceholder.targetLocation =
                  lapPoints[playerPlaceholder.nextPathIndex];
              }
            } else {
              playerPlaceholder.targetLocation = randomPositionInRadius(
                startPoint,
                radius
              );
            }
          }
          playerPlaceholder.position = newPosition;
          teamsPlaceholder[i].players[j] = playerPlaceholder;
        }
      }
    }

    setTeams(teamsPlaceholder);
  };

  const getNewPosition = (position, velocity) => {
    let newPosition = {
      lat: position.lat + velocity.lat,
      lng: position.lng + velocity.lng,
    };
    return newPosition;
  };

  const teamMarkers = () => {
    return teams.map((team, idx) =>
      team.players.map((player, index) => (
        <Marker
          key={idx + "-" + index}
          position={player.position}
          style={{ color: "white" }}
          label={idx + ", " + (index + 1)}
          color="red"
        />
      ))
    );
  };

  return isLoaded ? (
    <div>
      <div>
        <div>
          <FormControlLabel
            control={
              <Switch
                checked={sendGpsSignal}
                onChange={(e) => setSendGpsSignal(e.target.checked)}
              />
            }
            label="Send posisjon til api"
          />
        </div>
        <div>
          <Button variant="contained" onClick={() => buttonClick(0)}>
            Start
          </Button>
          <Button
            variant="contained"
            style={{ marginLeft: "10px" }}
            onClick={() => console.log(teams)}
          >
            Test
          </Button>
        </div>
      </div>
      <GoogleMap
        mapContainerStyle={containerStyle}
        center={startPoint}
        zoom={defaultZoom}
        options={{
          gestureHandling: "greedy",
          zoomControl: true,
          mapTypeControl: true,
          scaleControl: false,
          streetViewControl: false,
          rotateControl: true,
          fullscreenControl: true,
        }}
      >
        <Circle center={startPoint} radius={radiusMeter} />
        {teamMarkers()}
        {lapPoints.map((point, idx) => {
          return (
            <Marker
              position={point}
              key={idx}
              icon={{
                url: "https://maps.gstatic.com/mapfiles/ridefinder-images/mm_20_gray.png",
              }}
            />
          );
        })}
      </GoogleMap>
    </div>
  ) : (
    <div>Not Loaded</div>
  );
};

export default MovingPlayers;
